* Generic session options - page and line size, no page breaks;
OPTIONS nonumber nodate nocenter pagesize=MAX linesize=MAX formdlim=' ' spool;                                          
* Kill default titles on output, ODS tables in log, turn on graphics; 
TITLE; ODS TRACE OFF; *ODS LISTING; ODS GRAPHICS ON;

********************************************************************************;
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 4a               *******;
*******             CHANGE "filesave" to your folder path                *******;
********************************************************************************;

* Location for original files for these models  change this path;
* \\Client\ precedes path in Virtual Desktop outside H drive;
%LET filesave=C:\Dropbox\23_PSQF6270\PSQF6270_Example4a; 
LIBNAME filesave "&filesave.";

* Import Example 4a SAS data into work library;
DATA work.Example4a; SET filesave.PSQF6270_Example4a;
* Label existing variables for analysis;
  LABEL NLIvSLI=     "Group: 0=NLI, 1=SLI" 
        momed12=     "Mother Education (0=12 years")
        PropCorrect= "Grammar Test Proportion Correct";
* Create number correct for denominator of binomial outcome;
  Ntrials=100; 
  Ncorrect=ROUND(PropCorrect*Ntrials,1); 
* Compute number incorrect for zero-inflated binomial model;
  Nincorrect=Ntrials-Ncorrect; 
  PropIncorrect=1-PropCorrect;
RUN;

********************************************************************************;
******               BEGIN EXAMPLE 4a BINOMIAL MODELS                    *******;
********************************************************************************;

* Open external file to save results to;
ODS RTF FILE="&filesave.\PSQF6270_Example4a_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

TITLE "SAS Distribution of Proportion Correct";
PROC MEANS NDEC=2 DATA=work.Example4a; 
     VAR PropCorrect; 
RUN; 
PROC UNIVARIATE NOPRINT DATA=work.Example4a; 
     VAR PropCorrect; 
     HISTOGRAM PropCorrect / MIDPOINTS= 0 TO 1 BY .05 NORMAL(MU=EST SIGMA=EST); 
RUN; QUIT; TITLE;

TITLE "SAS Empty Binomial Model using GLIMMIX -- Ntrials is denominator";
PROC GLIMMIX DATA=work.Example4a NOCLPRINT NAMELEN=100 GRADIENT METHOD=MSPL;
     MODEL Ncorrect/Ntrials = / SOLUTION DDFM=NONE LINK=LOGIT DIST=BINOMIAL;
     ESTIMATE "Intercept" intercept 1 / ILINK; * ILINK gives intercept in probability;
RUN; TITLE;

TITLE "SAS Two-Predictor Binomial Model using GLIMMIX";
PROC GLIMMIX DATA=work.Example4a NOCLPRINT NAMELEN=100 GRADIENT METHOD=MSPL;
     MODEL Ncorrect/Ntrials = NLIvSLI momed12 
            / SOLUTION DDFM=NONE LINK=LOGIT DIST=BINOMIAL ODDSRATIO(LABEL);
     CONTRAST "Multiv Wald test of Model" NLIvSLI 1, momed12 1  / CHISQ;
RUN; TITLE;

TITLE1 "SAS Two-Predictor Binomial Model using GLIMMIX";
TITLE2 "Additive Overdispersion via Random Intercept Variance";
TITLE3 "Also known as observation-level random effect for overdispersion";
PROC GLIMMIX DATA=work.Example4a NOCLPRINT NAMELEN=100 GRADIENT METHOD=LAPLACE;
     CLASS ID;  * Person ID is added to CLASS because of RANDOM statement below;
     MODEL Ncorrect/Ntrials = NLIvSLI momed12 
             / SOLUTION DDFM=NONE LINK=LOGIT DIST=BINOMIAL ODDSRATIO(LABEL);
     CONTRAST "Multiv Wald test of Model" NLIvSLI 1, momed12 1 / CHISQ;
     RANDOM INTERCEPT / SUBJECT=ID;   * Add per-person "residual" as random intercept;
     COVTEST "Need Random Intercept Variance?" 0;  * LRT for additive overdispersion;
RUN; TITLE1; TITLE2; TITLE3;

/*
TITLE1 "SAS Two-Predictor Binomial Model";
TITLE2 "Switch to PROC FINITE MIXTURE MODEL to check LL";
PROC FMM DATA=work.Example4a NAMELEN=100;
     MODEL Ncorrect/Ntrials = NLIvSLI momed12 / LINK=LOGIT DIST=BINOMIAL;
     OUTPUT OUT=work.Binpred PRED=yhat VAR=yhatvar;
RUN; TITLE1; TITLE2; 
*/

TITLE1 "SAS Two-Predictor Beta-Binomial Model with Multiplicative Overdispersion";
TITLE2 "Using PROC FINITE MIXTURE MODEL that has beta-binomial distribution";
PROC FMM DATA=work.Example4a NAMELEN=100;
     MODEL Ncorrect/Ntrials = NLIvSLI momed12 / LINK=LOGIT DIST=BETABINOMIAL;
     OUTPUT OUT=work.BBpred PRED=yhat VAR=yhatvar;
RUN; TITLE1; TITLE2;
* Compute LRT for multiplicative overdispersion manually;
DATA work.LRTb; * Binomial vs beta-binomial;
     DevBin=1531.7332; DevBB=534.1033; 
     TestStat=DevBin-DevBB; TestDF=1;
     RegPvalue=1-PROBCHI(TestStat,TestDF);
     MixPvalue=(0.5*(1-PROBCHI(TestStat,TestDF)));
     FORMAT RegPValue MixPValue pvalue6.4; RUN;
TITLE1 "Likelihood Ratio Test for Binomial vs Beta-Binomial";
TITLE2 "Regular p-value uses DF, mixture p-value uses DF=DF,DF-1";
PROC PRINT NOOBS DATA=work.LRTb; RUN; TITLE1; TITLE2;
* Save corr of pred and actual propcorrect to square as R2;
TITLE "Correlation of Predicted and Actual PropCorrect";
PROC CORR NOSIMPLE DATA=work.BBpred OUT=work.Rpred; 
     VAR PropCorrect; WITH yhat; RUN; 
* Compute R2 in saved output;
DATA work.Rpred; SET work.Rpred;
     WHERE _TYPE_="CORR"; R2=PropCorrect*PropCorrect; RUN;
TITLE "R2 of Predicted and Actual PropCorrect";
PROC PRINT NOOBS DATA=work.Rpred; VAR R2; RUN; TITLE;

TITLE1 "SAS Two-Predictor Zero-Inflated Binomial Model";
TITLE2 "Using FMM and predicting Nincorrect instead";
PROC FMM DATA=work.Example4a NAMELEN=100;
     MODEL Nincorrect/Ntrials = NLIvSLI momed12 / LINK=LOGIT DIST=BINOMIAL;
     MODEL + / DIST=CONSTANT(0); * Empty inflation model predicting extra zero;
RUN; TITLE1; TITLE2;
* Compute LRT for zero-inflation manually;
DATA work.LRTc; * Binomial vs. zero-inflated binomial;
     DevBin=1531.7332; DevZBin=988.2183; 
     TestStat=DevBin-DevZBin; TestDF=1;
     RegPvalue=1-PROBCHI(TestStat,TestDF);
     MixPvalue=(0.5*(1-PROBCHI(TestStat,TestDF)));
     FORMAT RegPValue MixPValue pvalue6.4; RUN;
TITLE1 "Likelihood Ratio Test for Binomial vs Zero-Inflated Binomial";
TITLE2 "Regular p-value uses DF, mixture p-value uses DF=DF,DF-1";
PROC PRINT NOOBS DATA=work.LRTc; RUN; TITLE1; TITLE2;

TITLE1 "SAS Two-Predictor Zero-Inflated Beta-Binomial Model";
TITLE2 "Using FMM and predicting Nincorrect instead";
PROC FMM DATA=work.Example4a NAMELEN=100;
     MODEL Nincorrect/Ntrials = NLIvSLI momed12 / LINK=LOGIT DIST=BETABINOMIAL;
     MODEL + / DIST=CONSTANT(0); * Empty inflation model predicting extra zero;
     OUTPUT OUT=work.ZIBBpred PRED=yhat VAR=yhatvar;
RUN; TITLE1; TITLE2;
* Compute LRT for multiplicative overdispersion manually;
DATA work.LRTd; * Zero-inflated binomial vs beta-binomial;
     DevZBin=988.2183; DevZBB=527.5780;
     TestStat=DevZBin-DevZBB; TestDF=1;
     RegPvalue=1-PROBCHI(TestStat,TestDF);
     MixPvalue=(0.5*(1-PROBCHI(TestStat,TestDF)));
     FORMAT RegPValue MixPValue pvalue6.4; RUN;
TITLE1 "Likelihood Ratio Test for Zero-Inflated Binomial vs Beta-Binomial";
TITLE2 "Regular p-value uses DF, mixture p-value uses DF=DF,DF-1";
PROC PRINT NOOBS DATA=work.LRTd; RUN; TITLE1; TITLE2;
* Save corr of pred and actual propcorrect to square as R2;
TITLE "Correlation of Predicted and Actual PropIncorrect";
PROC CORR NOSIMPLE DATA=work.ZIBBpred OUT=work.Rpred; 
     VAR PropIncorrect; WITH yhat; RUN; 
* Compute R2 in saved output;
DATA work.Rpred; SET work.Rpred;
     WHERE _TYPE_="CORR"; R2=PropIncorrect*PropIncorrect; RUN;
TITLE "R2 of Predicted and Actual PropCorrect";
PROC PRINT NOOBS DATA=work.Rpred; VAR R2; RUN; TITLE;


* Close output;
ODS RTF CLOSE; 
